@@ -56,6 +56,13 @@ G_DEFINE_BOXED_TYPE (GgitCredSshInteractivePrompt,
 
 static void ggit_cred_ssh_interactive_initable_iface_init (GInitableIface  *iface);
 
+/**
+ * GgitCredSshInteractiveClass::prompt:
+ * @cred: a #GgitCredSshInteractive
+ * @prompts: (array length=num_prompts): a list of prompts.
+ * @num_prompts: number of prompts.
+  */
+
 G_DEFINE_TYPE_EXTENDED (GgitCredSshInteractive,
                         ggit_cred_ssh_interactive,
                         GGIT_TYPE_CRED,
@@ -186,7 +193,7 @@ callback_wrapper (const char                            *name,
 	g_free (wname);
 	g_free (winstruction);
 
-	GGIT_CRED_SSH_INTERACTIVE_GET_CLASS (interactive)->prompt (wprompts, num_prompts);
+	GGIT_CRED_SSH_INTERACTIVE_GET_CLASS (interactive)->prompt (interactive, wprompts, num_prompts);
 
 	for (i = 0; i < num_prompts; i++)
 	{
@@ -240,7 +247,7 @@ ggit_cred_ssh_interactive_initable_iface_init (GInitableIface *iface)
 
 GgitCredSshInteractive *
 ggit_cred_ssh_interactive_new (const gchar  *username,
-                         GError      **error)
+                               GError      **error)
 {
 	return g_initable_new (GGIT_TYPE_CRED_SSH_INTERACTIVE,
 	                       NULL, error,
