@@ -22,6 +22,7 @@
 #include "ggit-cred.h"
 #include "ggit-transfer-progress.h"
 #include "ggit-oid.h"
+#include "ggit-enum-types.h"
 
 #define GGIT_REMOTE_CALLBACKS_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GGIT_TYPE_REMOTE_CALLBACKS, GgitRemoteCallbacksPrivate))
 
@@ -30,16 +31,27 @@ struct _GgitRemoteCallbacksPrivate
 	git_remote_callbacks native;
 };
 
+enum
+{
+	PROGRESS,
+	TRANSFER_PROGRESS,
+	UPDATE_TIPS,
+	COMPLETION,
+	NUM_SIGNALS
+};
+
+static guint signals[NUM_SIGNALS] = {0,};
+
+
 /**
  * GgitRemoteCallbacksClass::credentials:
- * @callbacks:
- * @url:
- * @username_from_url:
- * @allowed_types:
- * @cred: (out):
- * @error:
+ * @callbacks: a #GgitRemoteCallbacks.
+ * @url: the url.
+ * @username_from_url: (allow-none): the username extracted from the url.
+ * @allowed_types: the allowed credential types.
+ * @error: a #GError for error reporting.
  *
- * Returns:
+ * Returns: (transfer full) (nullable): a #GgitCred or %NULL in case of an error.
  */
 
 G_DEFINE_TYPE (GgitRemoteCallbacks, ggit_remote_callbacks, G_TYPE_OBJECT)
@@ -61,75 +73,55 @@ ggit_remote_callbacks_class_init (GgitRemoteCallbacksClass *klass)
 
 	object_class->finalize = ggit_remote_callbacks_finalize;
 
+	signals[UPDATE_TIPS] =
+		g_signal_new ("update-tips",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (GgitRemoteCallbacksClass, update_tips),
+		              NULL, NULL,
+		              NULL,
+		              G_TYPE_NONE,
+		              3,
+		              G_TYPE_STRING,
+		              GGIT_TYPE_OID,
+		              GGIT_TYPE_OID);
+
+	signals[PROGRESS] =
+		g_signal_new ("progress",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (GgitRemoteCallbacksClass, progress),
+		              NULL, NULL,
+		              NULL,
+		              G_TYPE_NONE,
+		              1,
+		              G_TYPE_STRING);
+
+	signals[TRANSFER_PROGRESS] =
+		g_signal_new ("transfer-progress",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (GgitRemoteCallbacksClass, transfer_progress),
+		              NULL, NULL,
+		              NULL,
+		              G_TYPE_NONE,
+		              1,
+		              GGIT_TYPE_TRANSFER_PROGRESS);
+
+	signals[COMPLETION] =
+		g_signal_new ("completion",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (GgitRemoteCallbacksClass, completion),
+		              NULL, NULL,
+		              NULL,
+		              G_TYPE_NONE,
+		              1,
+		              GGIT_TYPE_REMOTE_COMPLETION_TYPE);
+
 	g_type_class_add_private (object_class, sizeof (GgitRemoteCallbacksPrivate));
 }
 
-static int
-progress_wrap (const char *str, int len, void *data)
-{
-	GgitRemoteCallbacks *self = GGIT_REMOTE_CALLBACKS (data);
-
-	if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->progress != NULL)
-	{
-		GError *error = NULL;
-
-		if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->progress (self,
-		                                                      str,
-		                                                      len,
-		                                                      &error))
-		{
-			return GIT_OK;
-		}
-		else
-		{
-			if (error)
-			{
-				giterr_set_str (GIT_ERROR, error->message);
-				g_error_free (error);
-			}
-
-			return GIT_ERROR;
-		}
-	}
-	else
-	{
-		return GIT_OK;
-	}
-}
-
-static int
-completion_wrap (git_remote_completion_type type, void *data)
-{
-	GgitRemoteCallbacks *self = GGIT_REMOTE_CALLBACKS (data);
-
-	if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->completion != NULL)
-	{
-		GError *error = NULL;
-		GgitRemoteCompletionType rt = (GgitRemoteCompletionType)type;
-
-		if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->completion (self,
-		                                                        rt,
-		                                                        &error))
-		{
-			return GIT_OK;
-		}
-		else
-		{
-			if (error)
-			{
-				giterr_set_str (GIT_ERROR, error->message);
-				g_error_free (error);
-			}
-
-			return GIT_ERROR;
-		}
-	}
-	else
-	{
-		return GIT_OK;
-	}
-}
-
 static int
 credentials_wrap (git_cred     **cred,
                   const char    *url,
@@ -138,28 +130,25 @@ credentials_wrap (git_cred     **cred,
                   void          *data)
 {
 	GgitRemoteCallbacks *self = GGIT_REMOTE_CALLBACKS (data);
+	GgitRemoteCallbacksClass *cls = GGIT_REMOTE_CALLBACKS_GET_CLASS (self);
 
-	if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->credentials != NULL)
+	*cred = NULL;
+
+	if (cls->credentials != NULL)
 	{
 		GgitCred *mcred = NULL;
 		GError *error = NULL;
 
-		if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->credentials (self,
-		                                                         url,
-		                                                         username_from_url,
-		                                                         allowed_types,
-		                                                         &mcred,
-		                                                         &error))
+		mcred = cls->credentials (self,
+		                          url,
+		                          username_from_url,
+		                          allowed_types,
+		                          &error);
+
+		if (mcred != NULL)
 		{
-			if (mcred != NULL)
-			{
-				*cred = _ggit_native_release (mcred);
-				g_object_unref (mcred);
-			}
-			else
-			{
-				*cred = NULL;
-			}
+			*cred = _ggit_native_release (mcred);
+			g_object_unref (mcred);
 
 			return GIT_OK;
 		}
@@ -169,9 +158,13 @@ credentials_wrap (git_cred     **cred,
 			{
 				giterr_set_str (GIT_ERROR, error->message);
 				g_error_free (error);
-			}
 
-			return GIT_ERROR;
+				return GIT_ERROR;
+			}
+			else
+			{
+				return GIT_PASSTHROUGH;
+			}
 		}
 	}
 	else
@@ -180,43 +173,33 @@ credentials_wrap (git_cred     **cred,
 	}
 }
 
+
+static int
+progress_wrap (const char *str, int len, void *data)
+{
+	GgitRemoteCallbacks *self = GGIT_REMOTE_CALLBACKS (data);
+	gchar *message;
+
+	message = g_strndup (str, len);
+
+	g_signal_emit (self, signals[PROGRESS], 0, message);
+
+	g_free (message);
+	return GIT_OK;
+}
+
 static int
 transfer_progress_wrap (const git_transfer_progress *stats, void *data)
 {
 	GgitRemoteCallbacks *self = GGIT_REMOTE_CALLBACKS (data);
+	GgitTransferProgress *p;
 
-	if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->transfer_progress != NULL)
-	{
-		GgitTransferProgress *p;
-		GError *error = NULL;
-		gint ret;
+	p = _ggit_transfer_progress_wrap (stats);
 
-		p = _ggit_transfer_progress_wrap (stats);
+	g_signal_emit (self, signals[TRANSFER_PROGRESS], 0, p);
+	ggit_transfer_progress_free (p);
 
-		if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->transfer_progress (self,
-		                                                               p,
-		                                                               &error))
-		{
-			ret = GIT_OK;
-		}
-		else
-		{
-			if (error)
-			{
-				giterr_set_str (GIT_ERROR, error->message);
-				g_error_free (error);
-			}
-
-			ret = GIT_ERROR;
-		}
-
-		ggit_transfer_progress_free (p);
-		return ret;
-	}
-	else
-	{
-		return GIT_OK;
-	}
+	return GIT_OK;
 }
 
 static int
@@ -226,45 +209,29 @@ update_tips_wrap (const char    *refname,
                   void          *data)
 {
 	GgitRemoteCallbacks *self = GGIT_REMOTE_CALLBACKS (data);
+	GgitOId *na;
+	GgitOId *nb;
 
-	if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->update_tips != NULL)
-	{
-		GgitOId *na;
-		GgitOId *nb;
-		gint ret;
-		GError *error = NULL;
+	na = _ggit_oid_wrap (a);
+	nb = _ggit_oid_wrap (b);
 
-		na = _ggit_oid_wrap (a);
-		nb = _ggit_oid_wrap (b);
+	g_signal_emit (self, signals[UPDATE_TIPS], 0, refname, na, nb);
 
-		if (GGIT_REMOTE_CALLBACKS_GET_CLASS (self)->update_tips (self,
-		                                                         refname,
-		                                                         na,
-		                                                         nb,
-		                                                         &error))
-		{
-			ret = GIT_OK;
-		}
-		else
-		{
-			if (error)
-			{
-				giterr_set_str (GIT_ERROR, error->message);
-				g_error_free (error);
-			}
+	ggit_oid_free (na);
+	ggit_oid_free (nb);
 
-			ret = GIT_ERROR;
-		}
+	return GIT_OK;
+}
 
-		ggit_oid_free (na);
-		ggit_oid_free (nb);
+static int
+completion_wrap (git_remote_completion_type type, void *data)
+{
+	GgitRemoteCallbacks *self = GGIT_REMOTE_CALLBACKS (data);
+	GgitRemoteCompletionType rt = (GgitRemoteCompletionType)type;
 
-		return ret;
-	}
-	else
-	{
-		return GIT_OK;
-	}
+	g_signal_emit (self, signals[COMPLETION], 0, rt);
+
+	return GIT_OK;
 }
 
 static void
@@ -277,18 +244,19 @@ ggit_remote_callbacks_init (GgitRemoteCallbacks *self)
 	self->priv->native = gcallbacks;
 
 	self->priv->native.sideband_progress = progress_wrap;
-	self->priv->native.completion = completion_wrap;
-	self->priv->native.credentials = credentials_wrap;
 	self->priv->native.transfer_progress = transfer_progress_wrap;
 	self->priv->native.update_tips = update_tips_wrap;
+	self->priv->native.completion = completion_wrap;
+
+	self->priv->native.credentials = credentials_wrap;
 
 	self->priv->native.payload = self;
 }
 
-git_remote_callbacks
+git_remote_callbacks *
 _ggit_remote_callbacks_get_native (GgitRemoteCallbacks *self)
 {
-	return self->priv->native;
+	return &self->priv->native;
 }
 
 /* ex:set ts=8 noet: */
